#!/usr/bin/perl
#
# (c) 2004 Tilman Linneweh
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the above copyright notice is
# retained.
#
# Getting started
# ###############
#
# Define a class rssitems in your stylesheet.
#
# $Id: rssfetch,v 1.24 2004/05/20 21:57:57 tilman Exp $

use vars qw/ $opt_v $opt_V $opt_h $opt_o $opt_d $opt_c $opt_e $opt_f $opt_c $opt_F /;
use strict;
use XML::RSS::Parser;
use LWP::UserAgent;
use Getopt::Std;
use Data::Dumper;
use Config::Simple;

# version variables
my $major = 0;
my $minor = 3;
my $micro = 0;

my ($prog) = ($0 =~ /([^\/]+)$/);
sub usage {
	print STDERR <<EOF;
usage: $prog [-cdefhvV]
        -c Number	Number of topics displayed
	-e Emailaddress	Useragent From address
	-F URL		feed
	-f filename	configurationfile
        -c comment	Useragent comment
	-o filename	outputfile
	-d		debugoutput
	-h		summary of command line options
	-v		verbose output
	-V		print the version and exit
EOF
	exit 0;
}

sub version {
	print "$prog version $major.$minor.$micro\n";
	exit $major;
}

my $comment = 'http://arved.narf.at/weblog/';
my $numberofitems = 3;
my $output = "/usr/local/www/data/rssfeeds";
my $emailaddress;
my $useragent = LWP::UserAgent->new;
my $debug;
my $verbose;
my $version;
my $content;
my $configfile;
my %feeds;

# Parse Commandline arguments
getopts('dhvV:o:c:e:f:c:F:');
&usage if $opt_h;
&version if $opt_V;
$verbose = 1 if $opt_v;
$debug = 1 if $opt_d;
$output = $opt_o if $opt_o;
$comment = $opt_c if $opt_c;
$emailaddress = $opt_e if $opt_e;
$numberofitems = $opt_c if $opt_c;
$configfile = $opt_f if $opt_f;
%feeds = ( '01' => $opt_F,) if $opt_F;
Config::Simple->import_from($configfile, \%feeds) if $opt_f;

$useragent->agent("rssfetch $major.$minor.$micro - $comment \n");
$useragent->from($emailaddress);
$useragent->env_proxy;
$useragent->timeout(10);

print "Starting rssfetch with verbose output\n" if $verbose;
print "Starting rssfetch with debug output\n" if $debug;
print "Outputfile: ".$output."\n" if $verbose;

open( FILE, ">$output.new" ) or die "can't open '$output.new': $!";

for ( sort keys %feeds ) {
    # create new instance of XML::RSS::Parser
    my $parser = new XML::RSS::Parser;

    # retrieve feed
    print "retriving feed ".$feeds{$_}."\n" if $verbose;
    my $url = URI->new( $feeds{$_} );
    my $response = $useragent->get($url);

    #Check for empty value
    if ($response->is_success) {
	$content = $response->content;
	print "request successfull\n" if $verbose;
    } else {
        print "request was not successfull\n" if $verbose;
        print $response->content if $debug;
	next;
    }
   
    print $content if $debug;
    #Fix broken feeds
    #$content =~ s/&amp;uuml;/ue/g;
    #$content =~ s/&amp;auml;/ae/g;
    #$content =~ s/&amp;ouml;/oe/g;
    #$content =~ s/&amp;deg;//g;
    #$content =~ s/\//g;
    #$content =~ s/&nbsp;/ /g;
    $content =~ s/< /&lt; /g;
    #$content =~ s/\.> /&gt; /g;
    #$content =~ s/&/&amp;/g;
    #$content =~ s/&quot;/"/g;
    #$content =~ s/\x80/&euro;/g;
    $content =~ s/baronesse de pix .*</baronesse de pix</;
 
    #parse the RSS content
    my $feed = new XML::RSS::Parser::Feed; 
    $feed = $parser->parse($content);
    print "Feed has ".$feed->item_count()." items. \n" if $verbose;
    if ($feed->item_count() == 0) {
        next;
    }
    # print the HTML channel
    print_feed($feed);
    if ($debug == 1) {
	my $datadump = Data::Dumper->new([$feed]);
        print $datadump ->Dump."\n\n";
    }
}

close FILE;
rename "$output.new", "$output" or print "Rename failed: $!";
#  or die "can't rename '$output.new' to '$output': $!";

sub print_feed {
    my ( $feed ) = @_;
    print "Processing Feed\n" if $verbose;
    my $title = XML::RSS::Parser->ns_qualify('title',$feed->rss_namespace_uri);
    my $link = XML::RSS::Parser->ns_qualify('link',$feed->rss_namespace_uri);

#    return unless defined($feed->channel->element($title)) && defined($feed->channel->element($link));
    print "Parsing Channeltitle..." if $verbose;

    my $channeltitle = $feed->channel->element($title)->value;
    my $channellink = $feed->channel->element($link)->value;
    print "done!\n" if $verbose;
    
    # Hack around stupid titles
    print "Fixing title..." if $verbose;
    $channeltitle=~ s/Wibble/lauri\'s diary/;
    $channeltitle=~ s/anorganic.org/jule - anorganic.org/;
    $channeltitle=~ s/VERFLIXT UND ZUGENEWST ! /Godany - verflixt und zugenewst! /;
    $channeltitle=~ s/Diary of a \(clever\?\) sheep./lofi - Diary of a sheep/;
    $channeltitle=~ s/ü/\&uuml\;/g;
    $channeltitle=~ s/ä/\&auml\;/g; 
    $channeltitle=~ s/Trying/friedolin - Trying/;
    $channeltitle=~ s/^Apos/friedolin - Apos/;
    $channeltitle=~ s/The Lunatic Fringe/Tim Pritlove - The Lunatic Fringe/;
    $channeltitle=~ s/a preponderance/Jonas Luster - a preponderance/;
    if ($channeltitle =~ /FreeBSD Project News/) {
         print "..FreeBSD feed!.." if $verbose;
         $title = 'title';
         $link = 'link';
    }
    print "done!\n" if $verbose;
    my $channeltarget= $channeltitle;
    $channeltarget=~ s/ //g;
    $channeltarget=~ s/!//g;
    $channeltarget=~ s/'//g;
    print FILE qq(<a href="$channellink" target="$channeltarget">$channeltitle</a>
<div class="rssitems">);
    my $i = 1;
    print "Writing ".$numberofitems." items to file.." if $verbose;
    foreach my $item ( @{$feed->items} ) {
       # check for null elements
       if (not defined($item->element($title))) {
           print "null title.." if $verbose;
	   next;
       }
       if (not defined($item ->element($link))) {
           print "null link.." if $verbose;
           next;
       }

       #debug output
       print $i.".." if $verbose;
       if ( $debug == 1 ) {
           foreach ( keys %{ $item->element } ) {
               next unless defined($item->element($_));
               print $_.": ".$item->element($_)->value."\n";
           }
       }

       my $topiclink = $item->element($link)->value;
       my $topictitle = $item->element($title)->value;
       # In XML & has to be encoded
       $topiclink =~ s/\&/\&amp;/g;
       $topictitle =~ s/ö/\&ouml\;/;
       $topictitle =~ s/ø/\&oslash\;/;
       $topictitle =~ s/ü/\&uuml\;/;
       $topictitle=~ s/ä/\&auml\;/g;
       $topictitle=~ s/\x9f/&szlig\;/g;
       $topictitle=~ s/ü/\&uuml\;/g;
       print FILE qq(<a href=\"$topiclink\" target=\"$i\">$topictitle</a><br />\n);
       $i++;
       #Stop cycling over items
       if ($i == ($numberofitems + 1)) {
            last;
       }
    }
    print "done!\n" if $verbose;
    print FILE qq(</div>\n);
}
